@@ -2,6 +2,25 @@ open! Base
 open! Stdio
 open! Expect_test_helpers_base
 
+let%expect_test "[am_running_expect_test] and [assert_am_running_expect_test] when true" =
+  require [%here] (am_running_expect_test ());
+  assert_am_running_expect_test [%here];
+  [%expect {| |}]
+;;
+
+let%test_unit "[am_running_expect_test] and [assert_am_running_expect_test] when false" =
+  assert (not (am_running_expect_test ()));
+  match assert_am_running_expect_test [%here] with
+  | () -> assert false
+  | exception exn ->
+    [%test_result: string]
+      (hide_positions_in_string (Exn.to_string exn))
+      ~expect:
+        "(\"This code should be run inside an expect test; currently, it is running \
+         outside an expect test.\"\n\
+        \  lib/expect_test_helpers/base/test/test_expect_test_helpers_base.ml:LINE:COL)"
+;;
+
 let%expect_test "multiple calls to [print_s] create multiple lines" =
   print_s [%message "hello"];
   print_s [%message "there"];
@@ -229,6 +248,68 @@ let%expect_test "[require_compare_equal] failure with [~message]" =
     ("The sky is falling!" 1 2) |}]
 ;;
 
+let%expect_test "[require_not_equal] success" =
+  require_not_equal [%here] (module Int) ~cr:Comment 1 2;
+  [%expect {||}]
+;;
+
+let%expect_test "[require_not_equal] failure" =
+  require_not_equal [%here] (module Int) ~cr:Comment 1 1;
+  [%expect
+    {|
+    (* require-failed: lib/expect_test_helpers/base/test/test_expect_test_helpers_base.ml:LINE:COL. *)
+    ("values are equal" 1 1) |}]
+;;
+
+let%expect_test "[require_not_equal] failure with [~message]" =
+  require_not_equal [%here] (module Int) ~cr:Comment 1 1 ~message:"The sky is falling!";
+  [%expect
+    {|
+    (* require-failed: lib/expect_test_helpers/base/test/test_expect_test_helpers_base.ml:LINE:COL. *)
+    ("The sky is falling!" 1 1) |}]
+;;
+
+let%expect_test "[require_not_equal] failure with [~if_false_then_print_s]" =
+  require_not_equal
+    [%here]
+    (module Int)
+    ~cr:Comment
+    1
+    1
+    ~if_false_then_print_s:(lazy [%message "The sky is falling!"]);
+  [%expect
+    {|
+    (* require-failed: lib/expect_test_helpers/base/test/test_expect_test_helpers_base.ml:LINE:COL. *)
+    ("values are equal" 1 1 "The sky is falling!") |}]
+;;
+
+let%expect_test "[require_compare_not_equal] success" =
+  require_compare_not_equal [%here] (module Int) ~cr:Comment 1 2;
+  [%expect {||}]
+;;
+
+let%expect_test "[require_compare_not_equal] failure" =
+  require_compare_not_equal [%here] (module Int) ~cr:Comment 1 1;
+  [%expect
+    {|
+    (* require-failed: lib/expect_test_helpers/base/test/test_expect_test_helpers_base.ml:LINE:COL. *)
+    ("values are equal" 1 1) |}]
+;;
+
+let%expect_test "[require_compare_not_equal] failure with [~message]" =
+  require_compare_not_equal
+    [%here]
+    (module Int)
+    ~cr:Comment
+    1
+    1
+    ~message:"The sky is falling!";
+  [%expect
+    {|
+    (* require-failed: lib/expect_test_helpers/base/test/test_expect_test_helpers_base.ml:LINE:COL. *)
+    ("The sky is falling!" 1 1) |}]
+;;
+
 let%expect_test "[require_does_not_raise], no exception" =
   require_does_not_raise [%here] ~hide_positions:true (fun () -> ());
   [%expect {| |}]
@@ -424,15 +505,10 @@ let%expect_test "hide_temp_files_in_string" =
 ;;
 
 let%expect_test "[require_sets_are_equal] success" =
-  require_sets_are_equal
-    [%here]
-    (module Int)
-    (Set.empty (module Int))
-    (Set.empty (module Int));
+  require_sets_are_equal [%here] (Set.empty (module Int)) (Set.empty (module Int));
   [%expect {| |}];
   require_sets_are_equal
     [%here]
-    (module Int)
     (Set.of_list (module Int) [ 1; 2; 3 ])
     (Set.of_list (module Int) [ 3; 2; 1 ]);
   [%expect {| |}]
@@ -441,7 +517,6 @@ let%expect_test "[require_sets_are_equal] success" =
 let%expect_test "[require_sets_are_equal] failure" =
   require_sets_are_equal
     [%here]
-    (module Int)
     ~cr:Comment
     (Set.of_list (module Int) [ 1; 2 ])
     (Set.of_list (module Int) [ 2; 3 ]);
@@ -456,7 +531,6 @@ let%expect_test "[require_sets_are_equal] failure" =
 let%expect_test "[require_sets_are_equal] failure with extras only in first" =
   require_sets_are_equal
     [%here]
-    (module Int)
     ~cr:Comment
     (Set.of_list (module Int) [ 1; 2 ])
     (Set.of_list (module Int) [ 2 ]);
@@ -469,7 +543,6 @@ let%expect_test "[require_sets_are_equal] failure with extras only in first" =
 let%expect_test "[require_sets_are_equal] failure with extras only in second" =
   require_sets_are_equal
     [%here]
-    (module Int)
     ~cr:Comment
     (Set.of_list (module Int) [ 2 ])
     (Set.of_list (module Int) [ 2; 3 ]);
@@ -482,7 +555,6 @@ let%expect_test "[require_sets_are_equal] failure with extras only in second" =
 let%expect_test "[require_sets_are_equal] failure with names" =
   require_sets_are_equal
     [%here]
-    (module Int)
     (Set.of_list (module Int) [ 1; 2 ])
     (Set.of_list (module Int) [ 2; 3 ])
     ~cr:Comment
@@ -801,3 +873,127 @@ let%expect_test "remove_backtrace" =
   |> print_s;
   [%expect {| ((backtrace ("ELIDED BACKTRACE"))) |}]
 ;;
+
+let%expect_test "print_and_check_stringable" =
+  print_and_check_stringable
+    ~cr:Comment
+    [%here]
+    (module Int32)
+    Int32.[ min_value; minus_one; zero; one; max_value ];
+  [%expect {|
+    -2147483648
+    -1
+    0
+    1
+    2147483647 |}]
+;;
+
+let%expect_test "print_and_check_sexpable" =
+  print_and_check_sexpable
+    ~cr:Comment
+    [%here]
+    (module Int32)
+    Int32.[ min_value; minus_one; zero; one; max_value ];
+  [%expect {|
+    -2147483648
+    -1
+    0
+    1
+    2147483647 |}]
+;;
+
+let%expect_test "print_and_check_round_trip" =
+  (* conversion works for all int64s *)
+  let module Int64_as_string = struct
+    type t = int64
+    type repr = string [@@deriving sexp_of]
+
+    let to_repr = Int64.to_string
+    let of_repr = Int64.of_string
+    let repr_name = "string"
+  end
+  in
+  (* conversion fails to round-trip for some int64s *)
+  let module Int64_as_int32_trunc = struct
+    type t = int64
+    type repr = int32 [@@deriving sexp_of]
+
+    let to_repr = Int64.to_int32_trunc
+    let of_repr = Int64.of_int32
+    let repr_name = "int32"
+  end
+  in
+  (* conversion raises for some int64s *)
+  let module Int64_as_int63_exn = struct
+    type t = int64
+    type repr = Int63.t [@@deriving sexp_of]
+
+    let to_repr x =
+      try Int63.of_int64_exn x with
+      | Failure string ->
+        (* error message varies by platform, so we normalize it *)
+        failwith (String.substr_replace_all string ~pattern:" int " ~with_:" int63 ")
+    ;;
+
+    let of_repr = Int63.to_int64
+    let repr_name = "int63"
+  end
+  in
+  (* test all three conversions *)
+  let test list =
+    print_and_check_round_trip
+      ~cr:Comment
+      [%here]
+      (module Int64)
+      [ (module Int64_as_string)
+      ; (module Int64_as_int32_trunc)
+      ; (module Int64_as_int63_exn)
+      ]
+      list
+  in
+  (* successful round-trip *)
+  test [ -1L; 0L; 1L ];
+  [%expect
+    {|
+    ((string -1)
+     (int32  -1)
+     (int63  -1))
+    ((string 0)
+     (int32  0)
+     (int63  0))
+    ((string 1)
+     (int32  1)
+     (int63  1)) |}];
+  (* unsuccessful round-trip *)
+  test
+    [ Int64.pred (Int64.of_int32 Int32.min_value)
+    ; Int64.succ (Int64.of_int32 Int32.max_value)
+    ];
+  [%expect
+    {|
+    ((string -2147483649)
+     (int32  2147483647)
+     (int63  -2147483649))
+    (* require-failed: lib/expect_test_helpers/base/test/test_expect_test_helpers_base.ml:LINE:COL. *)
+    ("int32 serialization failed to round-trip"
+      (original        -2147483649)
+      (int32           2147483647)
+      (int32_roundtrip 2147483647))
+    ((string 2147483648)
+     (int32  -2147483648)
+     (int63  2147483648))
+    (* require-failed: lib/expect_test_helpers/base/test/test_expect_test_helpers_base.ml:LINE:COL. *)
+    ("int32 serialization failed to round-trip"
+      (original        2147483648)
+      (int32           -2147483648)
+      (int32_roundtrip -2147483648)) |}];
+  (* conversion raises *)
+  test [ Int64.min_value; Int64.max_value ];
+  [%expect
+    {|
+    (* require-failed: lib/expect_test_helpers/base/test/test_expect_test_helpers_base.ml:LINE:COL. *)
+    ("unexpectedly raised" (
+      Failure
+      "conversion from int64 to int63 failed: -9223372036854775808 is out of range")) |}];
+  ()
+;;
